library(readr)
library(dplyr)
library(reticulate)
library(ggplot2)
library(purrr)
library(xtable)

#
#  Analysis of the aggregated data per actor behavior
#

perf <- read.csv(file = "output_BPIC17_performance_decomposed_forR.csv") |>
  group_by(source, sink) |> 
  mutate(duration_all_mean = mean(duration_hours[actor_context == 'all'])) |> 
  ungroup() |> 
  mutate(actor_context = factor(actor_context, 
                                levels = c("continuation", "interruption", "handover_idle", "handover_prioritized", "handover_deprioritized" )))

perf |> 
  count(source, sink)
perf |> 
  count(actor_context)
perf |>
  distinct(source, sink) |>
  nrow()
perf |> 
  filter(actor_context != 'all') |> 
  filter(duration_all_mean > 0) |>  # only reasonable work
  distinct(source, sink) |>
  nrow()

# How often does a behavioral mechanism dominate
perf |> 
  filter(actor_context != 'all') |> 
  filter(duration_all_mean > 0) |>  # only reasonable work
  mutate(context_dominates = actor_context_per > 0.9) |> 
  group_by(actor_context) |> 
  count(actor_context, context_dominates) |>  
  filter(context_dominates == T) |> 
  select(`Actor behavior` = actor_context, n) |> 
  xtable(type = "latex", label = "tab:mechanism_dominates", caption = "") |> 
  print(type = "latex", file = "mechanism_dominates.tex", booktabs = T, include.rownames = F)

# How often are multiple behavioral mechanism relevant
perf |> 
  filter(actor_context != 'all') |> 
  filter(duration_all_mean > 0) |>  # only reasonable work
  mutate(context_divided = actor_context_per < 0.5) |> 
  group_by(source, sink) |>
  summarise(all_divided = all(context_divided)) |> 
  ungroup() |> 
  count(all_divided) |> 
  xtable(type = "latex", label = "tab:mechanism_divided", caption = "") |> 
  print(type = "latex", file = "mechanism_divided.tex", booktabs = T)  

# What is the typical rank
perf |> 
  filter(actor_context != 'all') |>
  filter(duration_all_mean > 0) |>  # only reasonable work
  mutate(context_dominates = actor_context_per > 0.9) |> 
  mutate(context_represented = actor_context_per > 0.1) |> 
  mutate(context_present = actor_context_per > 0.0) |> 
  group_by(source, sink) |> 
  mutate(diff_hour_slower = duration_hours - duration_all_mean >= 1) |> 
  mutate(diff_workday_slower = duration_hours - duration_all_mean >= 8) |>
  mutate(diff_hour_faster = duration_hours - duration_all_mean <= -1) |> 
  mutate(diff_workday_faster = duration_hours - duration_all_mean <= -8) |>   
  mutate(rank_perf = dense_rank(duration_hours)) |> 
  mutate(rank_freq = dense_rank(desc(actor_context_cnt))) -> perf_ranks

perf_ranks |> 
  group_by(actor_context) |>
  summarise(`n` = sum(context_present), 
            `# >20%` = sum(context_represented),
            `# >90%` = sum(context_dominates),
            `# 1hr slower` = sum(diff_hour_slower),
            `# 1hr faster` = sum(diff_hour_faster),
            `# 8hr slower` = sum(diff_workday_slower),
            `# 8hr faster` = sum(diff_workday_faster),
            `Frequency rank` = mean(rank_freq),
            `Time rank` = mean(rank_perf)) |> 
  rename(behavior = actor_context) |> 
  xtable(type = "latex", label = "tab:mechanism_statistics", caption = "Summary statistics for the five behavioral mechanisms detected in the BPIC17 dataset.") |> 
  print(type = "latex", file = "mechanism_rank.tex", booktabs = T, include.rownames = F, table.placement = "tb", floating.environment = "table*")


perf |> 
  filter(actor_context != 'all') |> 
  filter((source == 'A_Concept-COMPLETE' & sink == 'A_Accepted-COMPLETE')
         | (source == 'W_Call incomplete files-SUSPEND' & sink == 'W_Call incomplete files-RESUME')
         | source == 'W_Validate application-SUSPEND' & sink == 'A_Denied-COMPLETE') |> 
  arrange(source, sink, actor_context) |> 
  mutate(edge = paste0(source, " < ", sink)) |> 
  mutate(edge = case_when(
    edge == 'A_Concept-COMPLETE < A_Accepted-COMPLETE' ~ 'AC+COMPLETE < AA+COMPLETE',
    edge == 'W_Call incomplete files-SUSPEND < W_Call incomplete files-RESUME' ~ 'WCIF+SUSPEND < WCIF+RESUME',
    edge == 'W_Validate application-SUSPEND < A_Denied-COMPLETE' ~ 'WVAP+SUSPEND < AD+COMPLETE',
    TRUE ~ edge
  )) |>   
  select(edge,
         behavior = actor_context,
         n = actor_context_cnt,
         `waiting time` = duration_hours,
         `# prioritized` = nr_prioritized,
         `# idle` = nr_idle) |>
  xtable(type = "latex", label = "tab:mechanism_edges", caption = "") |> 
  print(type = "latex", file = "mechanism_edges.tex", booktabs = T, include.rownames = F, table.placement = "tb", floating.environment = "table*")


#
#  Analysis of the raw instance data
#

# Load all raw data files (this needs reticulate and Python setup)
df_files <- dir("actor_behavior_all_instances_per_edge/", full.names = T, pattern = "*.pkl") 
map(df_files, 
    \(x) reticulate::py_load_object(x) |>
      mutate(df_edge = basename(x)) |> 
      bind_rows(dplyr::tibble(nr_prioritized=integer())) |> 
      select(df_edge, time, actor_context, nr_prioritized, duration_hours)) |>
  list_rbind() -> perf_raw

perf_raw |> 
  mutate(actor_context = factor(actor_context, 
                                levels = c("all", "continuation", "interruption", "handover_idle", "handover_prioritized", "handover_deprioritized" ),
                                labels = c("all", "C", "I", "HI", "HP", "HD"))) |> 
  group_by(df_edge) |> 
  mutate(duration_mean = mean(duration_hours[actor_context == 'all'])) |> 
  mutate(duration_perc = duration_hours / duration_mean) |> 
  mutate(duration_perc = if_else(is.na(duration_perc), 0, duration_perc)) -> perf_raw


# Figure showing all the instances
perf_raw |> 
  filter(duration_mean > 0 & actor_context != 'all') |>    # all non zero
  ggplot() +
  geom_boxplot(aes(duration_hours, actor_context), varwidth = F, lwd = 1) +
  xlim(c(0,240)) + # two weeks
  theme_bw(base_size = 24) +
  xlab("Waiting time (hours)") +
  ylab("Actor behavior") 
ggsave("perf_raw_nonzero_clipped_240hrs.pdf", device = "pdf", useDingbats = TRUE, height = 8, width = 15)
ggsave("perf_raw_nonzero_clipped_240hrs.png", device = "png", height = 5, width = 15)

perf_raw |> 
  filter(duration_hours > 0) |>    # all non zero
  mutate(df_edge = case_when(
    df_edge == 'actor_context_A_Concept-COMPLETE_A_Accepted-COMPLETE.pkl' ~ 'A_Concept-COMPLETE < A_Accepted-COMPLETE',
    df_edge == 'actor_context_W_Call incomplete files-SUSPEND_W_Call incomplete files-RESUME.pkl' ~ 'W_Call incomplete files-SUSPEND < W_Call incomplete files-RESUME',
    df_edge == 'actor_context_W_Validate application-SUSPEND_A_Denied-COMPLETE.pkl' ~ 'W_Validate application-SUSPEND < A_Denied-COMPLETE',
    TRUE ~ df_edge
  )) |> 
  mutate(df_edge = case_when(
    df_edge == 'A_Concept-COMPLETE < A_Accepted-COMPLETE' ~ '(AC, AA)',
    df_edge == 'W_Call incomplete files-SUSPEND < W_Call incomplete files-RESUME' ~ '(IFS, IFR)',
    df_edge == 'W_Validate application-SUSPEND < A_Denied-COMPLETE' ~ '(VAP, AD)',
    TRUE ~ df_edge
  )) -> perf_raw_for_facet  

# Figure showing three examples
perf_raw_for_facet |> 
  filter(df_edge == '(AC, AA)'
        | df_edge == '(IFS, IFR)'
        | df_edge == '(VAP, AD)') |> 
  ggplot() +
  geom_boxplot(aes(duration_hours, actor_context), varwidth = F, lwd=1) +
  xlim(c(0,240)) +
  theme_bw(base_size = 26) +
  xlab("Waiting time (hours)") +
  ylab("Actor behavior") +
  facet_grid(df_edge ~ .)
ggsave("perf_raw_individual_edges_facet.pdf", device = "pdf", height = 10, width = 15)
ggsave("perf_raw_individual_edges_facet.png", device = "png", height = 10, width = 15)
